@@ -18,13 +18,20 @@ import re
 import timeit
 
 import json
-from math import floor
 
-from time import time
-from collections.abc import Iterable
 from collections import deque
 from enum import Enum
+from . import data as siemkit_data
+from collections import ChainMap
 
+# ToDo: 1. Timeformats -> Have the time.py to either specify a format to parse, turn into a timestamp,
+#  or have the 'auto' string for using the timeparse library.
+# ToDo: 2. On the first with() statement where the Event() contains any outputs, start the sessions of all outputs
+#  by accessing their __enter__. On the most outter (or same level of initial outputs?),
+#  perform the __exit__ on all outputs.
+#   -Treat as sessions, thinking about writing to files or having a managed TCP connection.
+#   -Connectionless just don't manage anything, they just write on command.
+# ToDo: 3. Create a customizable output option. Make CSV files using those customized file types.
 
 # import dateparser  # This can't be good for performance. Need performance? know your formats!
 
@@ -37,7 +44,7 @@ from enum import Enum
 
 # ToDo: Manual Time Format with Declared fields as time -> Directed for performance
 # ToDo: Automatic Time Format detection -> Comfortable but requires an external library + performance penalty.
-
+# ToDo: Consider a syntax sugar for this: for _ in range(3000): _ = send.udp("127.0.0.1", 514, bytes(random_event(event)))  # Random certain aspects of the event.
 
 class CefSeverity(str, Enum):
 
@@ -94,6 +101,25 @@ class States:
         self.__states = list((dict() for _ in range(optimized_size)))
         self.__index = 0
 
+    def save(self, state):
+        """
+        Update initial root state.
+            This state is restored either by the `reset(self)` method or by one or more calls from the `restore(self)`.
+        :param state: A dictionary
+        :return:
+        """
+        self.__states[0].clear()
+        self.__states[0].update(state)
+
+    def reset(self):
+        """
+        Reset to initial root state
+        :return:
+        """
+        state_zero = self.__states[0]
+        self.clear()
+        return state_zero
+
     def store(self, state):
         if self.__index == self.__optimized_size:
             self.__states.append(dict())
@@ -119,9 +145,22 @@ class States:
         return self
 
 
-class AbstractEventFormat(dict):
+class EventFormat(dict):
+
     __aliases = {}
 
+    __default_aliases = {}
+
+    __default_keys = set()
+
+    @classmethod
+    def default_keys(cls):
+        return set(cls.__default_keys)
+
+    @classmethod
+    def default_aliases(cls):
+        return dict(cls.__default_keys)
+
     @staticmethod
     def key_assertion(key):
         return True
@@ -240,34 +279,10 @@ class AbstractEventFormat(dict):
     def from_timestamp():
         pass
 
-    # ToDo: Allow later assignment of 'output'
-    # ToDo: save() & restore() using stack.
-    def __init__(
-            self,
-            format_,
-            version,
-            headers,
-            data={},
-            raw=b'',
-            aliases={},
-            key_declaration=set(),
-            warnings=False,
-            key_assertion=None,
-            value_assertaion=None,
-            deserializer=None,
-            serializer=None,
-            syslog_header=None,
-            timestamp_fields=set(),
-            to_timestamp=None,
-            from_timestamp=None,
-            allow_empty_keys=False,
-            outputs=None,
-            tcp=None,
-            udp=None,
-            file=None,
-            size_limit=1024,
-            optimized_save_levels=5
-    ):
+    def __init__(self, format_, version, headers, data=None, raw=b'', aliases=None, fields=None, warnings=False,
+                 key_assertion=None, value_assertion=None, deserializer=None, serializer=None, syslog_header=None,
+                 timestamp_fields=None, to_timestamp=None, from_timestamp=None, allow_empty_keys=False, outputs=None,
+                 tcp=None, udp=None, file=None, size_limit=1024, optimized_state_levels=5):
 
         """
             format_             - Format name, as will be presented in serialized form (CEF, LEEF, etc.)
@@ -276,8 +291,10 @@ class AbstractEventFormat(dict):
             data                - Assign values using dictionary object
             raw                 - Unparsed raw event
             aliases             - Aliases dictionary for key access
-            key_declaration     - If declared, only these keys will be allowed.
-                                    Exceptions message will tip for close matches
+            fields     - Declare field names for optional enforcement, spelling tests, correctness & other
+                                    possible assistance.
+                                    [Move to flag -> If declared, only these keys will be allowed.
+                                    Exceptions message will tip for close matches]
             warnings            - Warn about bad usage / practices
             key_assertion       - Assertion function to test the key's creation / access correctness
             value_assertion     - Assertion function to test the value's assignment correctness
@@ -295,24 +312,37 @@ class AbstractEventFormat(dict):
             size_limit          - The event size limit. In order to avoid potential size exploits.
         """
 
+        super().__init__()
+        if timestamp_fields is None:
+            timestamp_fields = set()
+
+        if data is None:
+            data = {}
+
         if key_assertion is None:
-            key_assertion = AbstractEventFormat.key_assertion
+            key_assertion = EventFormat.key_assertion
         self.__key_assertion = key_assertion
 
         if deserializer is None:
-            deserializer = AbstractEventFormat.deserializer
+            deserializer = EventFormat.deserializer
         self.__deserializer = deserializer
 
         if serializer is None:
-            serializer = AbstractEventFormat.serializer
+            serializer = EventFormat.serializer
         self.__serializer = serializer
 
         if syslog_header is None:
-            syslog_header = AbstractEventFormat.syslog_header
+            syslog_header = EventFormat.syslog_header
         self.__syslog_header = syslog_header
 
+        if aliases is None:
+            aliases = {}
+
+        if fields is None:
+            fields = set()
+
         # Keep the dictionary outside of the object's scope in order to avoid paradox when setting/getting data.
-        AbstractEventFormat.__aliases[id(self)] = aliases
+        EventFormat.__aliases[id(self)] = aliases
 
         # ToDo: If 'restrict_keys', allow /assert only access to the given keys. -> replaced with 'assert_key' function
         # If given a wrong one, throw missing key/attribute exception.
@@ -341,10 +371,13 @@ class AbstractEventFormat(dict):
         self.__headers_order = headers  # Important to know the order
         self.__headers_hash_set = set(headers)  # Much faster to test against.
         self.__aliases = aliases
+        self.__fields = fields
+        self.__fields.update(siemkit_data.words_set(aliases))
+
         # self.__headers_set = set(headers)
 
         # After all values are set for the first time, use them as default for the `clear()` command.
-        self.__states = States(optimized_size=optimized_save_levels)
+        self.__states = States(optimized_size=optimized_state_levels)
         self.__states.store(self)
         #self.__default_state = {}
         #self.__default_state.update(self)
@@ -361,6 +394,14 @@ class AbstractEventFormat(dict):
         self.__output = None
         self.output(outputs)
 
+    def update_aliases(self, aliases):
+        EventFormat.__aliases[id(self)].update(aliases)
+        return self
+
+    def assign_aliases(self, aliases):
+        EventFormat.__aliases[id(self)] = aliases
+        return self
+
     def output(self, outputs):
 
         if outputs is not None and not any(
@@ -425,8 +466,8 @@ class AbstractEventFormat(dict):
         if '__' in key:
             key = key.replace('__', ' ')
 
-        if key in AbstractEventFormat.__aliases[self_id]:
-            key = AbstractEventFormat.__aliases[self_id][key]
+        if key in EventFormat.__aliases[self_id]:
+            key = EventFormat.__aliases[self_id][key]
 
         if ignore_exception:
             return super().get(key, None)
@@ -447,17 +488,18 @@ class AbstractEventFormat(dict):
     def __set(self, key, value):
         # super().__setattr__(key, value) # Value is an object in the heap. We just set a reference here.
         # Do not set attribute, that way we enforce the calling of __getattr__
-        self.__detected_changes = True
+        self.__detected_changes = True  # Restoring state means editing.
+        self.__commit_context = True
         self_id = id(self)
-        if key in AbstractEventFormat.__aliases[self_id]:
-            key = AbstractEventFormat.__aliases[self_id][key]
+        if key in EventFormat.__aliases[self_id]:
+            key = EventFormat.__aliases[self_id][key]
         super().__setitem__(key, value)
         return self
 
     def __setattr__(self, name, value):
 
         # Don't update private variables in the dictionary.
-        if name.startswith("_AbstractEventFormat_"):
+        if name.startswith("_EventFormat_"):
             return super().__setattr__(name, value)
 
         # If attribute has double underscores, treat as a single space.
@@ -482,38 +524,79 @@ class AbstractEventFormat(dict):
 
     def __enter__(self):
         self.__commit_context = True
-        self.save()
+        self.store()
         return self
 
     def __exit__(self, type_, value, traceback):
         if traceback:
             raise
 
-        # Done: Move this logic to write(self, object/collection of objects or self.__output)
         if self.__commit_context:
             if self.__output:
                 self.write()
 
             # We make sure that only the most inner & recent context is committed
             # to allow predicted behaviour
-            self.__commit_context = False
+            # self.__commit_context = False
 
         # ToDo: If a parse() method was activated, use super.clear() instead?
         # ToDo: Or use a different writing style for parsing:
 
         self.restore()
+        # We make sure that only the most inner & recent context is committed
+        # to allow predicted behaviour
+        self.__commit_context = False
+
+        # Todo: Check: if this is the last context, if so, close all outputs.
+        #             for output in self.__output:
+        #                   output.close()
+        return self
+
+    def abort(self):
+        """
+        Works only within most inner `with` statement (context manager).
+         Aborts output on exit for assigned values.
+        :return:
+        """
+        self.__commit_context = False
         return self
 
     # Set current values as default
     def save(self):
+        """
+        Set the current state as root state.
+        :return:
+        """
+        self.__states.save(self)
+        return self
+
+    def reset(self):
+        """
+        Reset current state to the saved root state.
+        :return:
+        """
+        state = self.__states.reset()
+        super().clear()
+        self.update(state)
+        return self  # Return to root state
+
+    def store(self):
+        """
+        Store current state.
+        :return:
+        """
         self.__states.store(self)
         return self
 
     def restore(self):
+        """
+        Restore previous state.
+        :return:
+        """
         state = self.__states.restore()
         super().clear()
         self.update(state)
-        return self  # Return to default state
+        return self  # Return to previous state
 
     # Done: Allow clearing all values, for parsing
     # Done: Or/And provide an automatic way for "clearing" when using the 'with' statement and receiving data
@@ -537,20 +620,165 @@ class AbstractEventFormat(dict):
     def close(self):  # Close all used resources -> Files, Sockets, etc.
         pass
 
+    def available_keys(self):
 
-class Cef(AbstractEventFormat):
+        self.__fields.update(self.keys())
+        return set(self.__fields)
+
+
+class Cef(EventFormat):
+
+    # REF: About Source/Attacker Destination/Target
+    # https://community.microfocus.com/t5/ArcSight-User-Discussions/Attacker-Address-versus-Source-Address/td-p/1582901
+    # To manually tie attacker and source fields together, assign aliases when creating an event object.
+    # e.g.  Cef(aliases={ "attackerAddress": "src" }) will tie attackerAddress to sourceAddress.
+    #  Cef(aliases={ "attackerAddress": "src" }) == Cef(aliases={ "attackerAddress": "sourceAddress"})
+    __default_aliases = {
+        'deviceAction': 'act',
+        'applicationProtocol': 'app',
+        'baseEventCount': 'cnt',
+        'eventOutcome': 'outcome',
+        'deviceAddress': 'dvc',
+        'deviceHostName': 'dvchost',
+        'deviceMacAddress': 'dvcmac',
+        'deviceProcessId': 'dvcpid',
+        'destinationAddress': 'dst',
+        'deviceTimeZone': 'dtz',
+        'destinationHostName': 'dhost',
+        'destinationMacAddress': 'dmac',
+        'destinationNtDomain': 'dntdom',
+        'destinationPort': 'dpt',
+        'destinationProcessName': 'dproc',
+        'destinationProcessId': 'dpid',
+        'destinationUserId': 'duid',
+        'destinationUserPrivileges': 'dpriv',
+        'destinationUserName': 'duser',
+        'endTime': 'end',
+        'fileName': 'fname',
+        'fileSize': 'fsize',
+        'bytesIn': 'in',
+        'message': 'msg',
+        'bytesOut': 'out',
+        'transportProtocol': 'proto',
+        'receiptTime': 'rt',
+        'deviceReceiptTime': 'rt',
+        'managerReceiptTime': 'mrt',
+        'requestUrl': 'request',
+        'requestURL': 'request',
+        'sourceAddress': 'src',
+        'sourceHostName': 'shost',
+        'sourceMacAddress': 'smac',
+        'sourcePort': 'spt',
+        'sourceUserPrivileges': 'spriv',
+        'sourceUserId': 'suid',
+        'sourceUserName': 'suser',
+        'sourceNtDomain': 'sntdom',
+        'sourceProcessId': 'spid',
+        'sourceProcessName': 'sproc',
+        'startTime': 'start',
+        'deviceEventCategory': 'cat',
+        'deviceCustomString1Label': 'cs1Label',
+        'deviceCustomString2Label': 'cs2Label',
+        'deviceCustomString3Label': 'cs3Label',
+        'deviceCustomString4Label': 'cs4Label',
+        'deviceCustomString5Label': 'cs5Label',
+        'deviceCustomString6Label': 'cs6Label',
+        'deviceCustomNumber1Label': 'cn1Label',
+        'deviceCustomNumber2Label': 'cn2Label',
+        'deviceCustomNumber3Label': 'cn3Label',
+        'deviceCustomString1': 'cs1',
+        'deviceCustomString2': 'cs2',
+        'deviceCustomString3': 'cs3',
+        'deviceCustomString4': 'cs4',
+        'deviceCustomString5': 'cs5',
+        'deviceCustomString6': 'cs6',
+        'deviceCustomNumber1': 'cn1',
+        'deviceCustomNumber2': 'cn2',
+        'deviceCustomNumber3': 'cn3',
+        'deviceCustomIPv6Address1': 'c6a1',
+        'deviceCustomIPv6Address1Label': 'c6a1Label',
+        'deviceCustomIPv6Address2': 'c6a2',
+        'deviceCustomIPv6Address2Label': 'c6a2Label',
+        'deviceCustomIPv6Address3': 'c6a3',
+        'deviceCustomIPv6Address3Label': 'c6a3Label',
+        'deviceCustomIPv6Address4': 'c6a4',
+        'deviceCustomIPv6Address4Label': 'c6a4Label',
+        'deviceCustomFloatingPoint1': 'cfp1',
+        'deviceCustomFloatingPoint1Label': 'cfp1Label',
+        'deviceCustomFloatingPoint2': 'cfp2',
+        'deviceCustomFloatingPoint2Label': 'cfp2Label',
+        'deviceCustomFloatingPoint3': 'cfp3',
+        'deviceCustomFloatingPoint3Label': 'cfp3Label',
+        'deviceCustomFloatingPoint4': 'cfp4',
+        'deviceCustomFloatingPoint4Label': 'cfp4Label',
+        'agentAddress': 'agt',
+        'agentHostName': 'ahost',
+        'agentId': 'aid',
+        'agentMacAddress': 'amac',
+        'agentReceiptTime': 'art',
+        'agentType': 'at',
+        'agentTimeZone': 'atz',
+        'agentVersion': 'av',
+        'destinationGeoLatitude': 'dlat',
+        'destinationGeoLongitude': 'dlong',
+        'sourceGeoLatitude': 'slat',
+        'sourceGeoLongitude': 'slong',
+    }
+
+    __default_keys = siemkit_data.words_set(__default_aliases)
+
+    __default_keys.update({
+        'attackerAddress',
+        'attackerHostName',
+        'attackerMacAddress',
+        'attackerPort',
+        'attackerUserPrivileges',
+        'attackerUserId',
+        'attackerUserName',
+        'attackerNtDomain',
+        'attackerProcessId',
+        'attackerProcessName',
+        'attackerGeoLatitude',
+        'attackerGeoLongitude'
+        'targetHostName',
+        'targetAddress',
+        'targetMacAddress',
+        'targetNtDomain',
+        'targetPort',
+        'targetProcessName',
+        'targetProcessId',
+        'targetUserId',
+        'targetUserPrivileges',
+        'targetUserName',
+        'targetGeoLatitude',
+        'targetGeoLongitude',
+        'deviceVendor',
+        'deviceProduct',
+        'deviceVersion',
+        'deviceEventClassId',
+        'name',
+        'deviceSeverity'
+    })
+
+    @classmethod
+    def default_keys(cls):
+        return set(cls.__default_keys)
+
+    @classmethod
+    def default_aliases(cls):
+        return dict(cls.__default_aliases)
 
     def __init__(
             self,
             version=0,
-            data={},
+            data=None,
             raw=b'',
-            aliases={},
-            key_declartion=set(),
+            aliases=None,
+            fields=None,
             key_assertion=None,
             deserializer=None,
             serializer=None,
-            timestamp_fields=set(),
+            timestamp_fields=None,
             to_timestamp=None,
             from_timestamp=None,
             allow_empty_keys=False,
@@ -560,10 +788,22 @@ class Cef(AbstractEventFormat):
             file=None
     ):
 
+        if fields is None:
+            fields = set()
+
+        if data is None:
+            data = {}
+
+        if aliases is None:
+            aliases = {}
+
+        if timestamp_fields is None:
+            timestamp_fields = set()
+
         cef_key_declaration = set()
 
         cef_json = {
-            'deviceVendor': 'CyberSIEM Community',
+            'deviceVendor': 'CyberSIEM(R) Community',
             'deviceProduct': 'SIEM Kit',
             'deviceVersion': '0',
             'deviceEventClassId': 100,
@@ -572,146 +812,35 @@ class Cef(AbstractEventFormat):
             'deviceSeverity': CefSeverity.UNKNOWN
         }
 
-        cef_key_declaration.update(cef_json.keys())
-
         cef_json.update(data)
 
-        # REF: About Source/Attacker Destination/Target
-        # https://community.microfocus.com/t5/ArcSight-User-Discussions/Attacker-Address-versus-Source-Address/td-p/1582901
-        # To manually tie attacker and source fields together, assign aliases when creating an event object.
-        # e.g.  Cef(aliases={ "attackerAddress": "src" }) will tie attackerAddress to sourceAddress
+        default_aliases = Cef.default_aliases()
+        cef_key_declaration.update(siemkit_data.words_set(aliases))
 
-        cef_aliases = {
-            'deviceAction': 'act',
-            'applicationProtocol': 'app',
-            'baseEventCount': 'cnt',
-            'eventOutcome': 'outcome',
-            'deviceAddress': 'dvc',
-            'deviceHostName': 'dvchost',
-            'deviceMacAddress': 'dvcmac',
-            'deviceProcessId': 'dvcpid',
-            'destinationAddress': 'dst',
-            'deviceTimeZone': 'dtz',
-            'destinationHostName': 'dhost',
-            'destinationMacAddress': 'dmac',
-            'destinationNtDomain': 'dntdom',
-            'destinationPort': 'dpt',
-            'destinationProcessName': 'dproc',
-            'destinationProcessId': 'dpid',
-            'destinationUserId': 'duid',
-            'destinationUserPrivileges': 'dpriv',
-            'destinationUserName': 'duser',
-            'endTime': 'end',
-            'fileName': 'fname',
-            'fileSize': 'fsize',
-            'bytesIn': 'in',
-            'message': 'msg',
-            'bytesOut': 'out',
-            'transportProtocol': 'proto',
-            'receiptTime': 'rt',
-            'deviceReceiptTime': 'rt',
-            'managerReceiptTime': 'mrt',
-            'requestUrl': 'request',
-            'requestURL': 'request',
-            'sourceAddress': 'src',
-            'sourceHostName': 'shost',
-            'sourceMacAddress': 'smac',
-            'sourcePort': 'spt',
-            'sourceUserPrivileges': 'spriv',
-            'sourceUserId': 'suid',
-            'sourceUserName': 'suser',
-            'sourceNtDomain': 'sntdom',
-            'sourceProcessId': 'spid',
-            'sourceProcessName': 'sproc',
-            'startTime': 'start',
-            'deviceEventCategory': 'cat',
-            'deviceCustomString1Label': 'cs1Label',
-            'deviceCustomString2Label': 'cs2Label',
-            'deviceCustomString3Label': 'cs3Label',
-            'deviceCustomString4Label': 'cs4Label',
-            'deviceCustomString5Label': 'cs5Label',
-            'deviceCustomString6Label': 'cs6Label',
-            'deviceCustomNumber1Label': 'cn1Label',
-            'deviceCustomNumber2Label': 'cn2Label',
-            'deviceCustomNumber3Label': 'cn3Label',
-            'deviceCustomString1': 'cs1',
-            'deviceCustomString2': 'cs2',
-            'deviceCustomString3': 'cs3',
-            'deviceCustomString4': 'cs4',
-            'deviceCustomString5': 'cs5',
-            'deviceCustomString6': 'cs6',
-            'deviceCustomNumber1': 'cn1',
-            'deviceCustomNumber2': 'cn2',
-            'deviceCustomNumber3': 'cn3',
-            'deviceCustomIPv6Address1': 'c6a1',
-            'deviceCustomIPv6Address1Label': 'c6a1Label',
-            'deviceCustomIPv6Address2': 'c6a2',
-            'deviceCustomIPv6Address2Label': 'c6a2Label',
-            'deviceCustomIPv6Address3': 'c6a3',
-            'deviceCustomIPv6Address3Label': 'c6a3Label',
-            'deviceCustomIPv6Address4': 'c6a4',
-            'deviceCustomIPv6Address4Label': 'c6a4Label',
-            'deviceCustomFloatingPoint1': 'cfp1',
-            'deviceCustomFloatingPoint1Label': 'cfp1Label',
-            'deviceCustomFloatingPoint2': 'cfp2',
-            'deviceCustomFloatingPoint2Label': 'cfp2Label',
-            'deviceCustomFloatingPoint3': 'cfp3',
-            'deviceCustomFloatingPoint3Label': 'cfp3Label',
-            'deviceCustomFloatingPoint4': 'cfp4',
-            'deviceCustomFloatingPoint4Label': 'cfp4Label',
-            'agentAddress': 'agt',
-            'agentHostName': 'ahost',
-            'agentId': 'aid',
-            'agentMacAddress': 'amac',
-            'agentReceiptTime': 'art',
-            'agentType': 'at',
-            'agentTimeZone': 'atz',
-            'agentVersion': 'av',
-            'destinationGeoLatitude': 'dlat',
-            'destinationGeoLongitude': 'dlong',
-            'sourceGeoLatitude': 'slat',
-            'sourceGeoLongitude': 'slong',
-        }
+        aliases.update(default_aliases)
 
-        for k, v in cef_aliases.items():
-            cef_key_declaration.add(k)
-            cef_key_declaration.add(v)
+
+        #for k, v in cef_aliases.items():
+        #    cef_key_declaration.add(k)
+        #    cef_key_declaration.add(v)
 
         # Done: Enable self (double) aliases
         for k, v in aliases.items():
-            if v in cef_aliases.keys():
-                cef_aliases[k] = cef_aliases[v]
+            if v in aliases.keys():
+                aliases[k] = aliases[v]
             else:
-                cef_aliases[k] = v
+                aliases[k] = v
         # cef_aliases.update(aliases)
 
         # Other Key Declaration
-        cef_key_declaration.update({
-            'attackerAddress',
-            'attackerHostName',
-            'attackerMacAddress',
-            'attackerPort',
-            'attackerUserPrivileges',
-            'attackerUserId',
-            'attackerUserName',
-            'attackerNtDomain',
-            'attackerProcessId',
-            'attackerProcessName',
-            'attackerGeoLatitude',
-            'attackerGeoLongitude'
-            'targetHostName',
-            'targetAddress',
-            'targetMacAddress',
-            'targetNtDomain',
-            'targetPort',
-            'targetProcessName',
-            'targetProcessId',
-            'targetUserId',
-            'targetUserPrivileges',
-            'targetUserName',
-            'targetGeoLatitude',
-            'targetGeoLongitude',
-        })
+        # cef_key_declaration.update()
+
+        # if default_aliases is None:
+        #    default_aliases = set(cef_key_declaration)
+
+        # Update potential custom key declarations
+        cef_key_declaration.update(cef_json.keys())
+        cef_key_declaration.update(fields)
 
         super().__init__(
             format_='CEF',
@@ -726,8 +855,8 @@ class Cef(AbstractEventFormat):
             ),
             data=cef_json,
             raw=raw,
-            aliases=cef_aliases,
-            key_declaration=cef_key_declaration,
+            aliases=aliases,
+            fields=cef_key_declaration,
             key_assertion=key_assertion,
             deserializer=deserializer,
             serializer=serializer,
@@ -742,7 +871,7 @@ class Cef(AbstractEventFormat):
         )
 
 
-class Leef(AbstractEventFormat):
+class Leef(EventFormat):
     pass
 
 
@@ -783,7 +912,7 @@ def test():
     event.hello = 'world'
 
     print(event.json())
-    event.save()
+    event.store()
 
     event.hello = 'nice!'
     event.src = "127.0.0.1"
