@@ -15,7 +15,21 @@
 from abc import ABC
 from abc import abstractmethod
 from enum import Enum
+from typing import Generator
+from typing import Tuple
+
+import socket
 
 
 class ReceiveInterface:
     pass
+
+
+def udp(ip_address, port, buffer_size=1024) -> Generator[Tuple[bytes, str], None, None]:
+
+    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+
+    sock.bind((ip_address, port))
+
+    while True:
+        yield sock.recvfrom(buffer_size)
