@@ -1,4 +1,4 @@
-#   Copyright (C) 2020 CyberSIEM (R)
+#   Copyright (C) 2020 CyberSIEM(R)
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -12,7 +12,10 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
+from typing import Any
+from collections import deque
 import datetime
+import re
 
 import pytimeparse
 # * pytimeparse - MIT License
@@ -95,5 +98,40 @@ def size(size_string: str) -> int:
 
 
 def boolean(bool_string: str) -> bool:
-    bool_string = bool_string.lower()
+
+    bool_string = bool_string.strip().lower()
     return bool_string in ('t', 'true', 'yes', 'ok', 'on', '1', 'some')
+
+
+def variable(var_string: str, var_dict: dict) -> Any:
+
+    assign = None
+
+    keys = var_string
+    if '=' in var_string:
+        m = re.match(r"^\s?([^=\s]+?)\s?=\s?(.*?)\s?$", var_string)
+        if m:
+            keys, assign = m.groups()
+
+    keys = deque(keys.split('.'))
+
+    while keys:
+        if assign is not None and len(keys) == 1:
+            if isinstance(var_dict, str):
+                var_dict = {var_dict: None}
+            var_dict[keys[0]] = assign
+            keys.clear()
+        else:
+            key = keys.popleft()
+            if key not in var_dict:
+                if isinstance(var_dict, str):
+                    var_dict = {var_dict: None}
+                var_dict[key] = {}
+
+            var_dict = var_dict.get(key)
+
+    if not assign and not var_dict:
+        return None
+
+    return assign or var_dict
+
