@@ -1,4 +1,4 @@
-#   Copyright (C) 2020 CyberSIEM (R)
+#   Copyright (C) 2020 CyberSIEM(R)
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -18,21 +18,47 @@ from random import choice
 
 from ipaddress import IPv4Address
 from typing import Generator
+from typing import Union
 from enum import EnumMeta
 
+import os
+import threading
+from time import time
+from math import floor
+
 from .const import DOMAINS
 from .const import NAMES
 from . import web
 from . import flag
 
 
+def safe_object_uuid(obj: object) -> str:
+    object_id = hex(id(obj))[2:]
+    process_id = hex(os.getpid())[2:]
+    timestamp = hex(floor(time() * 1e5))[2:]
+    thread_id = hex(threading.get_ident())[2:]
+    random_value = hex(getrandbits(32))[2:]
+
+    return f'{object_id}-{process_id}-{thread_id}-{timestamp}-{random_value}'
+
+
 def byte() -> int:
     return randint(0, 255)
 
 
-def compose_ip(amount: int = 1) -> Generator[IPv4Address, None, None]:
+def compose_ip(
+        from_address: Union[IPv4Address, str] = '0.0.0.0',
+        to_address: Union[IPv4Address, str] = '255.255.255.255',
+        amount: int = 1) -> Generator[IPv4Address, None, None]:
+
     for _ in range(amount):
-        yield IPv4Address(f"{byte()}.{byte()}.{byte()}.{byte()}")
+
+        yield IPv4Address(
+            randint(
+                int(IPv4Address(from_address)),
+                int(IPv4Address(to_address))
+            )
+        )
 
 
 def compose_domain(amount: int = 1) -> Generator[str, None, None]:
@@ -129,6 +155,15 @@ def compose_flag_value(*enums: EnumMeta, amount: int = 1, flags: int = 1) -> Gen
         yield random_flag
 
 
+def compose_port(from_port: int = 0, to_port: int = 65535, amount: int = 1) -> Generator[int, None, None]:
+
+    if not (0 <= from_port <= to_port <= 65535):
+        raise ValueError("Illegal port range. Legal port range 0-65535.")
+
+    for _ in range(amount):
+        yield randint(from_port, to_port)
+
+
 def enum_value(*enums: EnumMeta) -> object:
     return next(compose_enum_value(*enums, amount=1))
 
@@ -137,8 +172,25 @@ def flag_value(*enums: EnumMeta, flags=1) -> int:
     return next(compose_flag_value(*enums, flags=flags))
 
 
-def ip() -> IPv4Address:
-    return next(compose_ip())
+def ip(from_address: Union[IPv4Address, str] = '0.0.0.0',
+       to_address: Union[IPv4Address, str] = '255.255.255.255') -> IPv4Address:
+
+    return next(
+        compose_ip(
+            from_address=from_address,
+            to_address=to_address
+        )
+    )
+
+
+def port(from_port: int = 0, to_port: int = 65535) -> int:
+
+    return next(
+        compose_port(
+            from_port=from_port,
+            to_port=to_port
+        )
+    )
 
 
 def md5() -> str:
